<?php
// Переменные для настройки
$num_links_uv2 = 200;           // Количество ссылок УВ2 (на любой странице)
$num_links_uv3 = 100;           // Количество ссылок УВ3 (на страницах вида /X-"domain-zone")
$num_subdomains = 10;           // Количество поддоменов для вывода
$subdomain_length = 2;          // Количество случайных букв в поддомене
$link_file_path = __DIR__ . '/links.txt'; // Путь к файлу с ссылками

// Переменная с доменами, для которых не нужно делать редирект
$excluded_domains = ['site', 'site', 'site']; // Укажи домены через запятую

// Функция для записи логов с разделением на Googlebot и другие user-agent
function log_request() {
    $user_agent = $_SERVER['HTTP_USER_AGENT'];
    
    // Определяем, куда записывать логи в зависимости от user-agent
    $log_file = (stripos($user_agent, 'Googlebot') !== false) ? __DIR__ . '/log2.txt' : __DIR__ . '/log.txt';
    
    $date_time = date('Y-m-d H:i:s');
    $domain = $_SERVER['SERVER_NAME'];
    $request_uri = $_SERVER['REQUEST_URI'];

    // Строка для записи в лог
    $log_entry = "$date_time|$domain|$request_uri|$user_agent" . PHP_EOL;

    // Записываем лог в соответствующий файл
    file_put_contents($log_file, $log_entry, FILE_APPEND);
}

// Запись данных в лог
log_request();

// Функция для чтения файла с ссылками и возврата случайной ссылки
function get_random_link_from_file($file_path) {
    if (file_exists($file_path)) {
        $links = file($file_path, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        if (!empty($links)) {
            return $links[array_rand($links)]; // Возвращаем случайную строку из файла
        }
    }
    return null; // Возвращаем null, если файл пуст или не существует
}

// Функция для генерации карты сайта
function generate_sitemap($domain_zone) {
    header('Content-Type: application/xml; charset=utf-8');

    $sitemap = '<?xml version="1.0" encoding="UTF-8"?>' . PHP_EOL;
    $sitemap .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . PHP_EOL;

    // Генерация URL-ов
    for ($x = 1; $x <= 100; $x++) {
        for ($y = 1; $y <= 50; $y++) {
            $url = "https://" . $_SERVER['SERVER_NAME'] . "/$x-$y-$domain_zone";
            $sitemap .= "  <url>" . PHP_EOL;
            $sitemap .= "    <loc>$url</loc>" . PHP_EOL;
            $sitemap .= "  </url>" . PHP_EOL;
        }
    }

    $sitemap .= '</urlset>' . PHP_EOL;

    echo $sitemap;
    exit(); // Прекращаем дальнейшее выполнение скрипта
}

// Определение доменной зоны
$domain_zone = str_replace('.', '-', $_SERVER['SERVER_NAME']);

// Проверка URL, если запрашивается /sitemap.xml
if ($_SERVER['REQUEST_URI'] == '/sitemap.xml') {
    generate_sitemap($domain_zone);
}

// Проверка URL, если запрашивается /robots.txt
if ($_SERVER['REQUEST_URI'] == '/robots.txt') {
    header('Content-Type: text/plain');
    $rand_bot_name = generate_random_bot_name();
    $domain_name = $_SERVER['SERVER_NAME'];
    echo "User-agent: *\n";
    echo "Disallow: /\n\n";
    echo "User-Agent: GoogleBot\n";
    echo "Allow: /\n";
    echo "Sitemap: https://$domain_name/sitemap.xml\n\n";
    echo "User-Agent: Yandex\n";
    echo "Allow: /\n";
    echo "Crawl-delay: 10\n\n";
    echo "User-agent: $rand_bot_name Bot\n";
    echo "Disallow: /\n";
    exit(); // Прекращаем дальнейшее выполнение скрипта
}

// Функция для генерации случайной последовательности из 4 букв, где первая заглавная
function generate_random_bot_name() {
    $characters_upper = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $characters_lower = 'abcdefghijklmnopqrstuvwxyz';
    $bot_name = $characters_upper[rand(0, strlen($characters_upper) - 1)];
    for ($i = 0; $i < 3; $i++) {
        $bot_name .= $characters_lower[rand(0, strlen($characters_lower) - 1)];
    }
    return $bot_name;
}

// Функция для чтения CSV-файлов и возвращения массива фраз
function load_csv($file_path) {
    $lines = file($file_path, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
    return array_map('str_getcsv', $lines);
}

// Функция для обработки спиннеров {вариант1|вариант2|вариант3}
function process_spin($text) {
    return preg_replace_callback('/\{([^{}]*?)\}/', function($matches) {
        if (strpos($matches[1], '|') !== false) {
            $options = explode('|', $matches[1]);
            return $options[array_rand($options)];
        } else {
            return '{' . $matches[1] . '}';
        }
    }, $text);
}

// Функция для генерации случайного поддомена с указанной длиной
function generate_random_subdomain($length) {
    $characters = 'abcdefghijklmnopqrstuvwxyz';
    $subdomain = '';
    for ($i = 0; $i < $length; $i++) {
        $subdomain .= $characters[rand(0, strlen($characters) - 1)];
    }
    return $subdomain;
}

// Функция для определения домена второго уровня
function get_second_level_domain() {
    $parts = explode('.', $_SERVER['SERVER_NAME']);
    $domain = implode('.', array_slice($parts, -2));
    return $domain;
}

// Функция для генерации ссылок на поддомены третьего уровня
function generate_subdomain_links($domain, $num_links, $subdomain_length) {
    $links = '';
    for ($i = 0; $i < $num_links; $i++) {
        $subdomain = generate_random_subdomain($subdomain_length);
        $links .= "<li><a href=\"https://$subdomain.$domain\">https://$subdomain.$domain</a></li>";
    }
    return $links;
}

// Функция для генерации ссылок /X-"-domain-zone"
function generate_links($domain_zone, $num_links) {
    $links = '';
    for ($x = 1; $x <= $num_links; $x++) {
        $link_url = "/$x-$domain_zone";
        $links .= "<li><a href=\"$link_url\">$link_url</a></li>";  // Анкор в виде URL
    }
    return $links;
}

// Функция для генерации ссылок /X-Y-"-domain-zone"
function generate_uv3_links($x, $domain_zone, $num_links_uv3) {
    $links = '';
    for ($y = 1; $y <= $num_links_uv3; $y++) {
        $link_url = "/$x-$y-$domain_zone";
        $links .= "<li><a href=\"$link_url\">$link_url</a></li>";  // Анкор в виде URL
    }
    return $links;
}

// Функция для выполнения редиректа с вероятностью 20% для Googlebot с защитой через flock
function handle_redirect_for_googlebot($excluded_domains) {
    $user_agent = $_SERVER['HTTP_USER_AGENT'];
    $current_domain = $_SERVER['SERVER_NAME'];

    // Проверка, нужно ли пропустить редирект для текущего домена
    if (in_array($current_domain, $excluded_domains)) {
        return; // Пропускаем редирект, если домен в списке исключенных
    }

    if (stripos($user_agent, 'Googlebot') !== false) { // Проверка на наличие Googlebot в User-Agent
        if (rand(1, 100) <= 20) { // 20% вероятность
            $index_file = __DIR__ . '/index.txt';
            $sent_file = __DIR__ . '/sent.txt';

            // Попытка открыть файл index.txt
            $fp = fopen($index_file, 'r+');
            if ($fp && flock($fp, LOCK_EX)) { // Блокировка файла на запись
                $links = file($index_file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
                if (!empty($links)) {
                    $first_link = array_shift($links); // Получаем первую ссылку
                    
                    // Перезаписываем оставшиеся ссылки обратно в index.txt
                    ftruncate($fp, 0); // Очищаем содержимое файла
                    rewind($fp); // Возвращаем указатель в начало файла
                    fwrite($fp, implode(PHP_EOL, $links) . PHP_EOL);
                    flock($fp, LOCK_UN); // Снимаем блокировку
                    fclose($fp);

                    // Добавляем первую ссылку в sent.txt
                    file_put_contents($sent_file, $first_link . PHP_EOL, FILE_APPEND);

                    // Выполняем 301 редирект на первую ссылку
                    header("Location: $first_link", true, 301);
                    exit(); // Прекращаем дальнейшее выполнение скрипта
                }
                flock($fp, LOCK_UN); // Снимаем блокировку
            }
            if ($fp) {
                fclose($fp);
            }
        }
    }
}

// Загрузка данных из CSV
$keys_banks = array_map(function($a) { return $a[0]; }, load_csv(__DIR__ . '/data/keys-banks.csv'));
$keys_mfo = array_map(function($a) { return $a[0]; }, load_csv(__DIR__ . '/data/keys-mfo.csv'));

// Проверка, не является ли текущая страница главной
if ($_SERVER['REQUEST_URI'] != '/' && $_SERVER['REQUEST_URI'] != '/index.php') {
    handle_redirect_for_googlebot($excluded_domains); // Проверка редиректа только для Googlebot и не исключенных доменов
}

// Генерация заголовка и H1
$title_bank = $keys_banks[array_rand($keys_banks)];
$title_mfo = $keys_mfo[array_rand($keys_mfo)];
$title = "$title_bank - $title_mfo";
$h1 = $title_bank;

// Генерация контента для body
$text_files = ['text1.txt', 'text2.txt', 'text3.txt'];
$selected_text_file = $text_files[array_rand($text_files)];
$body_text = process_spin(file_get_contents(__DIR__ . '/' . $selected_text_file));
$body_text = str_replace('%key%', $h1, $body_text);

// Генерация контента для head
$head_lines = file(__DIR__ . '/head.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
$selected_head_lines = array_rand(array_flip($head_lines), rand(0, count($head_lines)));
$head_content = is_array($selected_head_lines) ? implode("\n", array_map('process_spin', $selected_head_lines)) : process_spin($selected_head_lines);

// Замена %key% в head.txt на значение H1
$head_content = str_replace('%key%', $h1, $head_content);

// Генерация ссылки на главную страницу с случайным анкором
$random_anchor = array_merge($keys_banks, $keys_mfo)[array_rand(array_merge($keys_banks, $keys_mfo))];
$home_link = "<a href=\"/\">$random_anchor</a>";

// Генерация 5 ссылок на поддомены третьего уровня (две случайные буквы)
$second_level_domain = get_second_level_domain();
$subdomain_links = generate_subdomain_links($second_level_domain, $num_subdomains, $subdomain_length);

// Проверка на URL вида /X-"-domain-zone"
$uv3_links = '';
if (preg_match("/^\/(\d+)-$domain_zone$/", $_SERVER['REQUEST_URI'], $matches)) {
    $x = (int)$matches[1];
    if ($x >= 1 && $x <= 100) {
        $uv3_links = generate_uv3_links($x, $domain_zone, $num_links_uv3);
    }
}

// Генерация списка ссылок /X-"-domain-zone" на любой странице
$links_list = generate_links($domain_zone, $num_links_uv2);

// Проверка на URL вида /X-Y-domain-zone и генерация ссылки на /X-domain-zone
$uv2_link = '';
if (preg_match("/^\/(\d+)-(\d+)-$domain_zone$/", $_SERVER['REQUEST_URI'], $matches)) {
    $x = (int)$matches[1];
    $uv2_link = "<p><a href=\"/$x-$domain_zone\">/$x-$domain_zone</a></p>";  // Анкор в виде URL
}

// Вероятность 1% на вывод ссылки из файла links.txt
$random_link = null;
if (rand(1, 100) === 1) {
    $random_link = get_random_link_from_file($link_file_path);
}

// Вывод сгенерированного HTML
?>
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title><?= htmlspecialchars($title) ?></title>
    <?= $head_content ?>
</head>
<body>
    <h1><?= htmlspecialchars($h1) ?></h1>
    <p><?= $body_text ?></p>
    
    <!-- Ссылка на главную страницу -->
    <p><?= $home_link ?></p>

    <?php if (!empty($uv2_link)): ?>
        <!-- Ссылка на страницу вида /X-"-domain-zone" -->
        <?= $uv2_link ?>
    <?php endif; ?>

    <!-- Вывод 100 ссылок вида /X-"-domain-zone" -->
    <ul>
        <?= $links_list ?>
    </ul>

    <!-- Вывод 100 ссылок вида /X-Y-"-domain-zone" если URL соответствует /X-"-domain-zone" -->
    <?php if (!empty($uv3_links)): ?>
        <ul>
            <?= $uv3_links ?>
        </ul>
    <?php endif; ?>

    <!-- Ссылки на поддомены третьего уровня (две случайные буквы) -->
    <ul>
        <?= $subdomain_links ?>
    </ul>

    <!-- Случайная ссылка с вероятностью 1% -->
    <?php if (!empty($random_link)): ?>
        <p><a href="<?= htmlspecialchars($random_link) ?>"><?= htmlspecialchars($random_link) ?></a></p>
    <?php endif; ?>
</body>
</html>
